﻿using System;
using System.Runtime.CompilerServices;

namespace TestProject
{
    /// <summary>
    /// 테스트 비동기 태스크 메소드 빌더
    /// </summary>
    public struct TestAsyncTaskMethodBuilder
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 비동기 태스크 메소드 빌더
        /// </summary>
        private AsyncTaskMethodBuilder builder;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 태스크 - Task

        /// <summary>
        /// 태스크
        /// </summary>
        public TestTask Task
        {
            get
            {
                return new TestTask(this.builder.Task);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성하기 - Create()

        /// <summary>
        /// 생성하기
        /// </summary>
        /// <returns>테스트 비동기 태스크 메소드 빌더</returns>
        public static TestAsyncTaskMethodBuilder Create()
        {
            return new TestAsyncTaskMethodBuilder { builder = AsyncTaskMethodBuilder.Create() };
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 결과 설정하기 - SetResult()

        /// <summary>
        /// 결과 설정하기
        /// </summary>
        public void SetResult()
        {
            this.builder.SetResult();
        }

        #endregion
        #region 예외 설정하기 - SetException(exception)

        /// <summary>
        /// 예외 설정하기
        /// </summary>
        /// <param name="exception">예되</param>
        public void SetException(Exception exception)
        {
            this.builder.SetException(exception);
        }

        #endregion

        #region 대기 완료시 처리하기 - AwaitOnCompleted<TAwaiter, TStateMachine>(awaiter, stateMachine)

        /// <summary>
        /// 대기 완료시 처리하기
        /// </summary>
        /// <typeparam name="TAwaiter">대기자 타입</typeparam>
        /// <typeparam name="TStateMachine">상태 머신 타입</typeparam>
        /// <param name="awaiter">대기자</param>
        /// <param name="stateMachine">상태 머신</param>
        public void AwaitOnCompleted<TAwaiter, TStateMachine>(ref TAwaiter awaiter, ref TStateMachine stateMachine) where TAwaiter : INotifyCompletion where TStateMachine : IAsyncStateMachine
        {
            this.builder.AwaitOnCompleted(ref awaiter, ref stateMachine);
        }

        #endregion
        #region 비관리 대기 완료시 처리하기 - AwaitUnsafeOnCompleted<TAwaiter, TStateMachine>(awaiter, stateMachine)

        /// <summary>
        /// 비관리 대기 완료시 처리하기
        /// </summary>
        /// <typeparam name="TAwaiter">대기자 타입</typeparam>
        /// <typeparam name="TStateMachine">상태 머신 타입</typeparam>
        /// <param name="awaiter">대기자</param>
        /// <param name="stateMachine">상태 머신</param>
        public void AwaitUnsafeOnCompleted<TAwaiter, TStateMachine>(ref TAwaiter awaiter, ref TStateMachine stateMachine) where TAwaiter : ICriticalNotifyCompletion where TStateMachine : IAsyncStateMachine
        {
            this.builder.AwaitUnsafeOnCompleted(ref awaiter, ref stateMachine);
        }

        #endregion
        #region 상태 머신 설정하기 - SetStateMachine(stateMachine)

        /// <summary>
        /// 상태 머신 설정하기
        /// </summary>
        /// <param name="stateMachine">상태 머신</param>
        public void SetStateMachine(IAsyncStateMachine stateMachine)
        {
            this.builder.SetStateMachine(stateMachine);
        }

        #endregion
        #region 시작하기 - Start<TStateMachine>(stateMachine)

        /// <summary>
        /// 시작하기
        /// </summary>
        /// <typeparam name="TStateMachine">상태 머신 타입</typeparam>
        /// <param name="stateMachine">상태 머신</param>
        public void Start<TStateMachine>(ref TStateMachine stateMachine) where TStateMachine : IAsyncStateMachine
        {
            this.builder.Start(ref stateMachine);
        }

        #endregion
    }
}