﻿using System;
using System.Collections.Generic;
using System.Runtime.CompilerServices;
using System.Threading;
using System.Threading.Tasks;

namespace TestProject
{
    /// <summary>
    /// 테스트 태스크
    /// </summary>
    [AsyncMethodBuilder(typeof(TestAsyncTaskMethodBuilder))]
    public class TestTask
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 스레드
        /// </summary>
        private Thread thread;

        /// <summary>
        /// 태스크
        /// </summary>
        private Task task;

        /// <summary>
        /// 완료 여부
        /// </summary>
        private bool isCompleted;

        /// <summary>
        /// 연결 액션 리스트
        /// </summary>
        private List<Action> continuationActionList = new List<Action>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 태스크 - Task

        /// <summary>
        /// 태스크
        /// </summary>
        public Task Task
        {
            get
            {
                return this.task;
            }
        }

        #endregion
        #region 완료 여부 - IsCompleted

        /// <summary>
        /// 완료 여부
        /// </summary>
        public bool IsCompleted
        {
            get
            {
                if(this.thread == null)
                {
                    return this.task.IsCompleted;
                }
                
                return this.isCompleted;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - TestTask(task)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="task">태스크</param>
        public TestTask(Task task)
        {
            this.task = task;
        }

        #endregion
        #region 생성자 - TestTask(action)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="action">액션</param>
        public TestTask(Action action)
        {
            this.thread = new Thread((ThreadStart)(() =>
            {
                action();

                this.isCompleted = true;

                foreach(Action continuationAction in this.continuationActionList)
                {
                    continuationAction();
                }
            }));

            this.thread.Start();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 연결 추가하기 - AddContinuation(action)

        /// <summary>
        /// 연결 추가하기
        /// </summary>
        /// <param name="action">액션</param>
        public void AddContinuation(Action action)
        {
            if(this.thread == null)
            {
                this.task.ContinueWith((t) => { action(); });
            }
            else
            {
                this.continuationActionList.Add(action);
            }
        }

        #endregion
        #region 대기자 구하기 - GetAwaiter()

        /// <summary>
        /// 대기자 구하기
        /// </summary>
        /// <returns>대기자</returns>
        public TestTaskAwaiter GetAwaiter()
        {
            TestTaskAwaiter awaiter = new TestTaskAwaiter(this);

            return awaiter;
        }

        #endregion
    }
}